package fpmon.chart;

import info.monitorenter.gui.chart.Chart2D;
import info.monitorenter.gui.chart.IAxis;
import info.monitorenter.gui.chart.IAxis.AxisTitle;
import info.monitorenter.gui.chart.IAxisTitlePainter;
import java.awt.Color;
import java.awt.Font;
import java.awt.Graphics;
import java.awt.geom.Rectangle2D;

/**
 * Draws the axis title at an arbitrary position.
 * 
 * @author Miles Asvachin
 */
public final class AxisTitlePainterPositioned implements IAxisTitlePainter
{
	private final double positionX;
	private final double positionY;
	private final double alignX;
	private final double alignY;
	
	/**
	 * @param newPositionX X position relative to the chart. 0 is on the left
	 * @param newPositionY Y position relative to the chart. 0 is at the bottom
	 * @param newAlignX which part of the label to treat as the origin. 0 is on the left
	 * @param newAlignY which part of the label to treat as the origin.
	 */
	public AxisTitlePainterPositioned(final double newPositionX, final double newPositionY, final double newAlignX, final double newAlignY)
	{
		positionX = newPositionX;
		positionY = newPositionY;
		alignX = newAlignX;
		alignY = newAlignY;
	}

	@Override
	public int getHeight(final IAxis axis, final Graphics g2d)
	{
		return 0;
	}
	
	@Override
	public int getWidth(final IAxis axis, final Graphics g2d)
	{
		return 0;
	}

	@Override
	public void paintTitle(final IAxis axis, final Graphics g)
	{
		final AxisTitle axisTitle = axis.getAxisTitle();
		final String title = axisTitle.getTitle();

		// manage the title font if there:
		final Font titleFont = axisTitle.getTitleFont();
		final Font backUpFont = g.getFont();
		final Color titleColor = axisTitle.getTitleColor();
		final Color backUpColor = g.getColor();

		if(titleFont != null)
		{
			g.setFont(titleFont);
		}
		if(titleColor != backUpColor && titleColor != null)
		{
			g.setColor(titleColor);
		}

		final Rectangle2D bounds = g.getFontMetrics().getStringBounds(title, g);

		final Chart2D chart = axis.getAccessor().getChart();

		final int startX = chart.getXChartStart();
		final int endX = chart.getXChartEnd();
		final double sizeX = bounds.getWidth();
		final int titleStartX = startX + (int)((endX - startX) * positionX) - (int)(sizeX * alignX);
		
		final int startY = chart.getYChartStart();
		final int endY = chart.getYChartEnd();
		final double sizeY = bounds.getHeight();
		final int titleStartY = startY + (int)((endY - startY) * positionY) - (int)(sizeY * alignY);
		
		g.drawString(title, titleStartX, titleStartY);

		// resetting original font if it was changed:
		if(titleFont != null)
		{
			g.setFont(backUpFont);
		}

		if(titleColor != backUpColor && titleColor != null)
		{
			g.setColor(backUpColor);
		}
	}
}
