package fpmon.chart;

import fpmon.utils.StringUtils;
import info.monitorenter.gui.chart.labelformatters.ALabelFormatter;
import java.text.DecimalFormat;

/**
 * Formats the labels to use between 1 and 3 digits and a binary prefix (KB, MB etc).
 * 
 * @author Miles Asvachin
 */
public final class LabelFormatterBytes extends ALabelFormatter
{
	/**
	 * How much the label value must be divided by so that it is in the correct units.
	 */
	private double divisor = 1;
	/**
	 * Unit denoting the magnitude of the label value.
	 */
	private String unit = "B";
	/**
	 * If true, then the unit is displayed as a rate, i.e. MB/s
	 */
	private final boolean isRate;
	/**
	 * When this is true, round label values to integers, otherwise display with
	 * 1 decimal place.
	 */
	private boolean round;

	/**
	 * @param rate if true, then the unit is displayed as a rate, i.e. MB/s
	 * instead of just MB.
	 */
	public LabelFormatterBytes(final boolean rate)
	{
		isRate = rate;
	}
	
	@Override
	public String format(final double value)
	{
		final double scaledValue = value / divisor;
		return round ? Long.toString(Math.round(scaledValue)) : new DecimalFormat("#.#").format(scaledValue);
	}
	
	@Override
	public int getMaxAmountChars()
	{
		return 4; //It's really 3, but the axis label is drawn too close then.
	}
	
	@Override
	public void initPaintIteration()
	{
		final double max = getAxis().getMax();
		final int exp = StringUtils.getByteSize((long)max);
		final String byteSize = StringUtils.getByteSize(exp);
		
		divisor = Math.pow(1024.0, exp);
		unit = isRate ? byteSize + "/s" : byteSize;
		
		round = (Math.round(max / divisor) >= 10);
		
		getAxis().getAxisTitle().setTitle(unit);
	}

	@Override
	public double getMinimumValueShiftForChange()
	{
		return 1.0;
	}

	@Override
	public double getNextEvenValue(double value, boolean ceiling)
	{
		return ceiling ? Math.ceil(value) : Math.floor(value);
	}

	@Override
	public Number parse(String formatted) throws NumberFormatException
	{
		return Double.parseDouble(formatted) * divisor;
	}
}
