package fpmon.chart;

import info.monitorenter.gui.chart.Chart2D;
import info.monitorenter.gui.chart.axis.AxisLinear;
import java.awt.Graphics;

/**
 * An x-axis that never draws its labels past the right hand edge, minimizing 
 * wasted space.
 * 
 * @author Miles Asvachin
 */
public final class XAxisNoOverhang extends AxisLinear
{
	@Override
	protected AChart2DDataAccessor createAccessor(final Chart2D chart, final int dimension, final int position)
	{
		final AChart2DDataAccessor result;
		
		if(dimension == Chart2D.X)
		{
			// Don't allow a combination of dimension and position that is not usable:
			if((position & (Chart2D.CHART_POSITION_BOTTOM | Chart2D.CHART_POSITION_TOP)) == 0)
			{
				throw new IllegalArgumentException("X axis only valid with top or bottom position.");
			}
			
			setAxisPosition(position);
			result = new XDataAccessorNoOverhang(chart);
		}
		else
		{
			throw new IllegalArgumentException("This class is meant to be an x-axis only!");
		}
		
		return result;
	}

	public final class XDataAccessorNoOverhang extends XDataAccessor
	{
		public XDataAccessorNoOverhang(final Chart2D chart)
		{
			super(chart);
		}
		
		@Override
		public int getWidth(final Graphics g2d)
		{
			return 0;
		}
	}
}
