package fpmon.stats;

import fpmon.utils.StringUtils;
import info.monitorenter.gui.chart.*;
import java.awt.Color;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.swing.BorderFactory;
import javax.swing.BoxLayout;
import javax.swing.JComponent;
import org.hyperic.sigar.Mem;
import org.hyperic.sigar.SigarException;
import org.hyperic.sigar.SigarProxy;

/**
 * Memory use.
 * 
 * @author Miles Asvachin
 */
public final class MemStatCollector extends ChartStatCollector
{
	private final ITrace2D lineTrace;

	{
		title = "Memory";
		traceColor = Color.GREEN;
	}

	public MemStatCollector(final JComponent container, final long updatePeriod, final SigarProxy sigar)
	{
		container.setLayout(new BoxLayout(container, BoxLayout.X_AXIS));
		container.setBorder(BorderFactory.createEmptyBorder(0, 0, 0, rightBorder));
		container.setBackground(backgroundColor);
		
		try
		{
			final Mem mem = sigar.getMem();
			final String memTotal = StringUtils.formatBytes(mem.getTotal());
			title += " (" + memTotal + ")";
		}
		catch(SigarException ex)
		{
			Logger.getLogger(MemStatCollector.class.getName()).log(Level.WARNING, "Couldn't get memory info", ex);
		}

		final Chart2D lineChart = createLineChart(title, backgroundColor, foregroundColor);
		axisTime(lineChart.getAxisX(), timeAxisFormat);
		axisPercent(lineChart.getAxisY());
		lineTrace = addLineTrace(lineChart, traceSize, title, traceColor, fillAlpha);
		container.add(lineChart);
		initLineTrace(lineTrace, updatePeriod);
	}
	
	@Override
	protected void update(final SigarProxy sigar)
	{
		try
		{
			final Mem mem = sigar.getMem();
			final double memPercent = mem.getUsedPercent();
			final ITracePoint2D point = new TracePoint2D(System.currentTimeMillis(), memPercent);
			lineTrace.addPoint(point);
			
			final String memUsed = StringUtils.formatBytes(mem.getActualUsed());
			final String traceName = String.format("Used: %s (%s%%)", memUsed, Math.round(memPercent));
			lineTrace.setName(traceName);
		}
		catch(SigarException ex)
		{
			Logger.getLogger(CpuStatCollector.class.getName()).log(Level.WARNING, "Couldn't get memory info", ex);
		}
	}
}
