package fpmon.utils;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.net.MalformedURLException;
import java.net.URL;
import java.nio.charset.Charset;
import java.util.logging.Level;
import java.util.logging.Logger;

/**
 * Gets this machine's IP address from whatismyip.org
 * 
 * @author Miles Asvachin
 */
public final class IpAddressFinder
{
	protected String address = "http://whatismyip.org";
	
	protected final URL url;
	protected String lastPublicIpAddress;
	protected String currentPublicIpAddress;
	
	public IpAddressFinder() throws MalformedURLException
	{
		url = new URL(address);
	}
	
	public String gePublicIpAddress()
	{
		lastPublicIpAddress = currentPublicIpAddress;

		try (final BufferedReader in = new BufferedReader(new InputStreamReader(url.openStream(), Charset.forName("UTF-8"))))
		{
			currentPublicIpAddress = in.readLine();
			
			if(currentPublicIpAddress != null)
			{
				final String[] splitAddress = currentPublicIpAddress.split("\\.");
				
				if(splitAddress.length == 4)
				{
					for(final String s : splitAddress)
					{
						final int i = Integer.parseInt(s);
						
						if(i > 249)
						{
							throw new NumberFormatException("IP address is not in the correct format: " + currentPublicIpAddress);
						}
					}
				}
				else
				{
					throw new NumberFormatException("IP address is not in the correct format: " + currentPublicIpAddress);
				}
			}
		}
		catch(IOException | NumberFormatException ex)
		{
			Logger.getLogger(IpAddressFinder.class.getName()).log(Level.WARNING, "Failed to retrieve public IP address", ex);
			currentPublicIpAddress = null;
		}

		return currentPublicIpAddress;
	}
	
	/**
	 * @return -1 if current and previous addresses are invalid. 0 if both 
	 * addresses are valid and equal. 1 if the last address was valid and the 
	 * current address is invalid (change for the worse). 2 if the last address 
	 * was invalid and the current address is valid (change for the better), 3
	 * if both addresses are valid but not equal.
	 */
	public int publicAddressState()
	{
		final int state;

		if(currentPublicIpAddress == null)
		{
			if(lastPublicIpAddress == null)
			{
				state = -1;
				Logger.getLogger(IpAddressFinder.class.getName()).log(Level.WARNING, "Possible loss of internet connectivity");
			}
			else
			{
				state = 1;
				Logger.getLogger(IpAddressFinder.class.getName()).log(Level.WARNING, "Possible loss of internet connectivity");
			}
		}
		else
		{
			if(lastPublicIpAddress == null)
			{
				state = 2;
				Logger.getLogger(IpAddressFinder.class.getName()).log(Level.INFO, "New public IP address: {0}", currentPublicIpAddress);
			}
			else
			{
				if(currentPublicIpAddress.equals(lastPublicIpAddress))
				{
					state = 0;
				}
				else
				{
					state = 3;
					Logger.getLogger(IpAddressFinder.class.getName()).log(Level.INFO, "Public IP address changed from {0} to {1}", new Object[]{lastPublicIpAddress, currentPublicIpAddress});
				}
			}
		}
		
		return state;
	}
}
